#include <Application.h>
#include <Alert.h>
#include <Bitmap.h>
#include <Button.h>
#include <Entry.h>
#include <File.h>
#include <FilePanel.h>
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <NodeInfo.h>
#include <Roster.h>
#include <Screen.h>
#include <ScrollView.h>
#include <String.h>
#include <StringView.h>
#include <View.h>

#include "PlayWindow.h"

PlayWindow::PlayWindow(BRect frame)
	: BWindow(frame, "Playlister", B_TITLED_WINDOW, B_NOT_ZOOMABLE),
		SavePanel(NULL), prefWinWasLaunched(false)
{
	// Load last window size/position
	BRect windowSize;
	if (prefs.FindRect("winsize", &windowSize) != B_OK)
		windowSize = frame;

	MoveTo(windowSize.left, windowSize.top);
	ResizeTo(windowSize.Width(), windowSize.Height());

	// Grey background view
	BRect body = Bounds();
	BView *background = new BView(body, NULL, B_FOLLOW_ALL_SIDES, 0);
	rgb_color grey = ui_color(B_PANEL_BACKGROUND_COLOR);
	background->SetViewColor(grey);
	background->SetLowColor(grey);
	AddChild(background);
	
	// Account for menu bar at top
	background->MoveBy(0, 18);
	body.bottom -= 18;		
	
	// Set minimum size
	SetSizeLimits(body.right, body.right*3, body.bottom, body.bottom*3); 
	
	// Menu bar at the top
	BRect menuBarRect = Bounds();
	menuBarRect.bottom = 18;
	MenuBar = new BMenuBar(menuBarRect, "TheMagicFantasticWonderfulMenuBar");
	AddChild(MenuBar);
	
	// -- File Menu
	FileMenu = new BMenu("File");
	FileMenu->AddItem(new BMenuItem("About Playlister", new BMessage(ABOUT)));
	FileMenu->AddItem(new BMenuItem("Preferences...", new BMessage(PREFS), 'P'));
	FileMenu->AddSeparatorItem();
	FileMenu->AddItem(new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED), 'Q'));
	MenuBar->AddItem(FileMenu);
	
	// -- Playlist Menu
	PlaylistMenu = new BMenu("Playlist");
	PlaylistMenu->AddItem(new BMenuItem("Move Up", new BMessage(MOVE_UP), B_UP_ARROW));
	PlaylistMenu->AddItem(new BMenuItem("Move Down", new BMessage(MOVE_DOWN), B_DOWN_ARROW));
	PlaylistMenu->AddItem(new BMenuItem("Remove", new BMessage(REMOVE)));
	PlaylistMenu->AddItem(new BMenuItem("Clear All", new BMessage(CLEAR), 'L'));	
	PlaylistMenu->AddSeparatorItem();
	PlaylistMenu->AddItem(new BMenuItem("Sort By Path", new BMessage(SORT_PATH), 'T'));
	PlaylistMenu->AddItem(new BMenuItem("Sort By Title", new BMessage(SORT_TITLE), 'T', B_SHIFT_KEY));
	PlaylistMenu->AddItem(new BMenuItem("Randomize", new BMessage(RANDOMIZE), 'R'));
	PlaylistMenu->AddSeparatorItem();
	PlaylistMenu->AddItem(new BMenuItem("Play Selected", new BMessage(PLAY), 'Y'));
	MenuBar->AddItem(PlaylistMenu); 
	
	// Listing of files to put in the playlist
	BRect r1(body);
	r1.InsetBy(10, 10);
	r1.bottom -= 40;
	r1.right -= B_V_SCROLL_BAR_WIDTH;
	SongList = new SongListView(r1);

	SongListScroll = new BScrollView("ScrollBoy", SongList, B_FOLLOW_ALL, 0, false, true);
	background->AddChild(SongListScroll);
	SongList->MakeFocus(true);
	
	// Icon
	BRect r2(0,0,31,31);
	r2.OffsetTo(10, body.bottom - 38);
	IconView = new BView(r2, NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	background->AddChild(IconView);

	Icon = new BBitmap(BRect(0,0,31,31), B_CMAP8);
	Icon->SetBits(IconBits, 32*32*8, 0, B_CMAP8);
	IconView->SetViewBitmap(Icon, B_FOLLOW_NONE);
	
	// Titles n' Stuff
	BRect r3(45, body.bottom-40, body.right-250, body.bottom-15);
	Title = new BStringView(r3, NULL, "Playlister 1.2");
	Title->SetFontSize(20);
	background->AddChild(Title);
	
	BRect r4(45, body.bottom-15, body.right-250, body.bottom-5);
	SubTitle = new BStringView(r4, NULL, "Automatic Playlist Generator");
	background->AddChild(SubTitle);
	
	// Buttons
	BRect r5(body.right-250, body.bottom-33, body.right-180, body.bottom-23);
	PlayButton = new BButton(r5, NULL, "Play", new BMessage(PLAY));
	PlayButton->SetEnabled(false);
	background->AddChild(PlayButton);
	
	BRect r6(body.right-160, body.bottom-33, body.right-90, body.bottom-23);
	CancelButton = new BButton(r6, NULL, "Cancel", new BMessage(CANCEL));
	background->AddChild(CancelButton);	
	
	BRect r7(body.right-80, body.bottom-33, body.right-10, body.bottom-23);
	SaveButton = new BButton(r7, NULL, "Save", new BMessage(SAVE));
	SaveButton->MakeDefault(true);
	SaveButton->SetEnabled(false);
	background->AddChild(SaveButton);
}

void PlayWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
		case PLAY:
		{
			bool somethingIsSelected = false;
			for (int32 i=0; i<SongList->CountItems(); i++)
				if (SongList->IsItemSelected(i))
					somethingIsSelected = true;
					
			// Save it in the temp directory, then launch it
			if (somethingIsSelected) {
				SaveSelected(BDirectory("/tmp"), "tmplaylist.m3u");
				char *filename = "/tmp/tmplaylist.m3u";
				be_roster->Launch(PLAYLIST_TYPE, 1, &filename);
			}				
		}
			break;
	
		case SAVE:
		{
			// Get the preferred playlist dir
			entry_ref savedir;
			const char *path;

			prefs.Load();
			if (prefs.FindString("playlistdir", &path) != B_OK)
				path = DEF_PLAYLIST_DIR;
			get_ref_for_path(path, &savedir);			

			if (SavePanel == NULL) {
				SavePanel = new BFilePanel(B_SAVE_PANEL, new BMessenger(this), &savedir, B_FILE_NODE, false);
				SavePanel->SetSaveText(".m3u");
			}
			SavePanel->Show();
		}
			break;
			
		case B_SAVE_REQUESTED:
		{
			RequestSave(msg);
			
			prefs.Load();
			bool closeWin = false;
			bool playList = false;

			// Optionally play playlist
			if (prefs.FindBool("playonsave", &playList) == B_OK && playList) {
				entry_ref dir;
				const char *name;
								
				msg->FindRef("directory", &dir);
				msg->FindString("name", &name);
				
				BPath path(&dir);
				path.Append(name);
				
				const char *fuckDoIHateConstChars = path.Path();
				char *playlist = const_cast<char *>(fuckDoIHateConstChars);
				be_roster->Launch(PLAYLIST_TYPE, 1, &playlist);
			}			

			// Optionally close the window
			if (prefs.FindBool("quitonsave", &closeWin) == B_OK && closeWin)
				PostMessage(B_QUIT_REQUESTED);	
		}	
			break;
		
		case CANCEL:
			PostMessage(B_QUIT_REQUESTED);
			break;
			
		case ABOUT:
			BAlert *About;
			About = new BAlert("about", "Playlister\n\nBy Syn.Terra\ndream@aevum.net", "Cool");
			About->Go();
			break;
			
		case PREFS:
		{
			float winWidth = 300;
			float winHeight = 145;
		
			BScreen theScreen;
			BRect prefBox = theScreen.Frame();
			float screenWidth = prefBox.right;
			float screenHeight = prefBox.bottom;
			
			prefBox.left = (screenWidth - winWidth) / 2;
			prefBox.top = (screenHeight - winHeight) / 2;
			prefBox.right = prefBox.left + winWidth;
			prefBox.bottom = prefBox.top + winHeight;
		
			PrefWin = new PrefWindow(prefBox, &prefs);
			prefWinWasLaunched = true;
		}		
			break;
			
		case CLEAR:			
		case MOVE_DOWN:
		case MOVE_UP:
		case REMOVE:
		case RANDOMIZE:
		case SORT_PATH:
		case SORT_TITLE:
			BMessenger(SongList, SongList->Window()).SendMessage(msg);
			break;			
			
		default:
			BWindow::MessageReceived(msg);
	}
}

bool PlayWindow::QuitRequested()
{
	// Save prefs
	prefs.SetRect("winsize", Frame());
	prefs.Save();

	// Clean up
	delete SavePanel;
	if (prefWinWasLaunched)
		PrefWin->PostMessage(B_QUIT_REQUESTED);
	
	return true;
}

void PlayWindow::FrameResized(float width, float height)
{
	Lock();
	SongListScroll->Draw(SongListScroll->Bounds());
	Unlock();
	
	height -= 18; 	// Account for the menu bar
	
	PlayButton->MoveTo(width-250, height-33);
	CancelButton->MoveTo(width-160, height-33);
	SaveButton->MoveTo(width-83, height-36);
	IconView->MoveTo(10, height-38);
	Title->MoveTo(45, height-40);
	SubTitle->MoveTo(45, height-15);

	BWindow::FrameResized(width, height);
}

status_t PlayWindow::RequestSave(BMessage *message)
{
	entry_ref ref;
	const char *name;
	status_t err;
	
	// Find the directory and filename
	if ((err = message->FindRef("directory", &ref)) != B_OK)
		return err;
	if ((err = message->FindString("name", &name)) != B_OK)
		return err;

	// Make a new BDirectory object, check if that went okay
	BDirectory dir(&ref);
	if ((err = dir.InitCheck()) != B_OK)
		return err;
	
	// Pass it to the Save function
	return Save(dir, name);
}

status_t PlayWindow::Save(BDirectory dir, const char *name)
{
	status_t err;
	
	// Create a new file, check if that was cool
	BFile file(&dir, name, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	if ((err = file.InitCheck()) != B_OK)
		return err;

	// Set the mimetype
	BNodeInfo info(&file);
	info.SetType(PLAYLIST_TYPE);

	// Write the playlist to a file
	BString data;
	for (int32 i=0; i<SongList->CountItems(); i++)
		data << ((BStringItem *)SongList->ItemAt(i))->Text() << "\n";
		
	err = file.Write(data.String(), data.Length());
	
	return err;
}

status_t PlayWindow::SaveSelected(BDirectory dir, const char *name)
{
	status_t err;
	
	// Create a new file, check if that was cool
	BFile file(&dir, name, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	if ((err = file.InitCheck()) != B_OK)
		return err;

	// Set the mimetype
	BNodeInfo info(&file);
	info.SetType(PLAYLIST_TYPE);

	// Write the playlist to a file
	BString data;
	for (int32 i=0; i<SongList->CountItems(); i++)
		if (SongList->IsItemSelected(i))
			data << ((BStringItem *)SongList->ItemAt(i))->Text() << "\n";
		
	err = file.Write(data.String(), data.Length());
	
	return err;
}

void PlayWindow::EnablePlayButton(bool how)
{
	PlayButton->SetEnabled(how);
}

void PlayWindow::EnableSaveButton(bool how)
{
	SaveButton->SetEnabled(how);
}

void PlayWindow::AddSongs(entry_ref ref)
{
	SongList->AddSongs(ref);
}

