#include <Application.h>
#include <Beep.h>
#include <Button.h>
#include <CheckBox.h>
#include <Entry.h>
#include <FilePanel.h>
#include <Path.h>
#include <StringView.h>
#include <TextControl.h>
#include <View.h>

#include "PrefWindow.h"

PrefWindow::PrefWindow(BRect frame, Preferences *oldPrefs)
	: BWindow(frame, "Playlister Preferences", B_FLOATING_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, B_NOT_RESIZABLE | B_NOT_ZOOMABLE),
		PlaylistPanel(NULL)
{
	// Load current preferences
	prefs = oldPrefs;
	prefs->Load();
	
	if (prefs->FindBool("quitonsave", &quitOnSave) != B_OK)
		quitOnSave = false;
		
	if (prefs->FindBool("playonsave", &playOnSave) != B_OK)
		playOnSave = false;
		
	if (prefs->FindString("playlistdir", &playlistDir) != B_OK)
		playlistDir = DEF_PLAYLIST_DIR;
	
	// Grey background view
	BRect body = Bounds();
	BView *background = new BView(body, NULL, B_FOLLOW_ALL_SIDES, 0);
	rgb_color grey = ui_color(B_PANEL_BACKGROUND_COLOR);
	background->SetViewColor(grey);
	background->SetLowColor(grey);
	AddChild(background);
	
	// Quit On Save Checkbox
	QuitOnSave = new BCheckBox(BRect(10, 10, body.right-10, 20), NULL, "Close Window After Saving", new BMessage(QUIT_ON_SAVE));
	if (quitOnSave) QuitOnSave->SetValue(B_CONTROL_ON);		
	background->AddChild(QuitOnSave);
	
	// Play On Save Checkbox
	PlayOnSave = new BCheckBox(BRect(10, 30, body.right-10, 40), NULL, "Play Playlist After Saving", new BMessage(PLAY_ON_SAVE));
	if (playOnSave) PlayOnSave->SetValue(B_CONTROL_ON);
	background->AddChild(PlayOnSave);
	
	// Playlist Dir Label
	BStringView *PlaylistDirLabel = new BStringView(BRect(7, 64, body.right-10, 74), NULL, "By Default, Playlists Are Saved In:");
	background->AddChild(PlaylistDirLabel);

	// Playlist Dir Text Control
	PlaylistDir = new BTextControl(BRect(5, 78, body.right-80, 88), NULL, NULL, playlistDir.String(), new BMessage(PLAYLIST_DIR));
	PlaylistDir->SetModificationMessage(new BMessage(PLAYLIST_DIR));
	background->AddChild(PlaylistDir);

	// Browse Button
	PlaylistButton = new BButton(BRect(body.right-70, 75, body.right-5, 85), NULL, "Browse...", new BMessage(PLAYLIST_BROWSE));
	background->AddChild(PlaylistButton);	
	
	// Save and Cancel buttons
	BRect r1(body.right-160, body.bottom-33, body.right-90, body.bottom-13);
	CancelButton = new BButton(r1, NULL, "Cancel", new BMessage(B_QUIT_REQUESTED));
	background->AddChild(CancelButton);
	
	BRect r2(body.right-80, body.bottom-33, body.right-10, body.bottom-13);
	SaveButton = new BButton(r2, NULL, "Save", new BMessage(SAVE_PREFS));
	SaveButton->MakeDefault(true);
	background->AddChild(SaveButton);
	
	Show();
}

void PrefWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what)
	{
		case SAVE_PREFS:
			prefs->SetBool("quitonsave", quitOnSave);
			prefs->SetBool("playonsave", playOnSave);
			prefs->SetString("playlistdir", playlistDir.String());
			prefs->Save();
			PostMessage(B_QUIT_REQUESTED);	
			break;
			
		case QUIT_ON_SAVE:
			if (QuitOnSave->Value() == B_CONTROL_ON)
				quitOnSave = true;
			else
				quitOnSave = false;
			break;
			
		case PLAY_ON_SAVE:
			if (PlayOnSave->Value() == B_CONTROL_ON)
				playOnSave = true;
			else
				playOnSave = false;
			break;
			
		case PLAYLIST_BROWSE:
			if (PlaylistPanel == NULL)
				PlaylistPanel = new BFilePanel(B_OPEN_PANEL, new BMessenger(this), NULL, B_DIRECTORY_NODE, false);
			PlaylistPanel->Show();	
			break;
			
		case B_REFS_RECEIVED:
		{
			entry_ref dir;
			msg->FindRef("refs", &dir);
			BPath path(&dir);
			PlaylistDir->SetText(path.Path());
		}
			break;
			
		case PLAYLIST_DIR:
			playlistDir = PlaylistDir->Text();
			break;
			
		default:
			BWindow::MessageReceived(msg);
	}
}

bool PrefWindow::QuitRequested()
{
	delete PlaylistPanel;
	
	return true;
}

	